require "lightspeed/configuration"
require "lightspeed/api/error"
require "lightspeed/api/tokens"
require "lightspeed/api/accounts"
require "lightspeed/api/sales"
require "lightspeed/api/customers"
require "lightspeed/api/items"
require "lightspeed/api/categories"

module Lightspeed
  def self.configuration
    @_configuration ||= Configuration.new
  end

  def self.configure
    yield configuration
  end

  class Client
    attr_reader :access_token, :account_id

    def self.tokens(client_id:, client_secret:, refresh_token:)
      API::Tokens.new(client_id, client_secret, refresh_token)
    end

    def initialize(access_token:, account_id:, configuration: Lightspeed::Configuration.new)
      @access_token = access_token
      @account_id = account_id
      @configuration = configuration
    end

    def get(url, params: {}, relations: [])
      response = conn.get(url, params.merge(load_relations: "[#{relation_names(relations)}]"))
      rate_limit_wait(response)
      response
    end

    def post(url, body: {})
      response = conn.post(url, body)
      rate_limit_wait(response)
      response
    end

    def delete(url)
      response = conn.delete(url)
      rate_limit_wait(response)
      response
    end

    def put(url, body: {})
      response = conn.put(url, body)
      rate_limit_wait(response)
      response
    end

    def accounts; API::Accounts.new(self); end
    def sales; API::Sales.new(self); end
    def customers; API::Customers.new(self); end
    def items; API::Items.new(self); end
    def categories; API::Categories.new(self); end

    private

    attr_reader :configuration

    def conn
      @conn ||= Faraday.new(Lightspeed.configuration.server) do |f|
        f.authorization :Bearer, access_token if access_token
        f.request :json
        f.response :json, :content_type => /\bjson$/
        f.adapter Faraday.default_adapter
      end
    end

    def relation_names(relations)
      relations.map do |relation|
        "\"#{relation.to_s.camelize}\""
      end.join(",")
    end

    def rate_limit_wait(response, points_expense: 10, points_threshold_percentage: 0.5)
      return unless Lightspeed.configuration.rate_limit || configuration.rate_limit

      bucket_level, bucket_limit = response.headers["x-ls-api-bucket-level"].split("/").map(&:to_f)
      drip_level = response.headers["x-ls-api-drip-rate"].to_f
      threshold = bucket_limit * points_threshold_percentage

      return if bucket_level + points_expense <= threshold

      points_over = bucket_level + points_expense - threshold
      wait_for_points = points_over / drip_level
      sleep(wait_for_points)
    end
  end
end
